@@ -1,4 +1,4 @@
-#' Retrieve from ANAC website all dates available for flights data
+#' Retrieve all dates available for flights data from ANAC website
 #'
 #' @return Numeric vector.
 #' @export
@@ -10,7 +10,7 @@
 get_flight_dates_available <- function() {
 
   # read html table
-  url = 'https://www.gov.br/anac/pt-br/assuntos/regulados/empresas-aereas/envio-de-informacoes/microdados/'
+  url = 'https://www.gov.br/anac/pt-br/assuntos/regulados/empresas-aereas/Instrucoes-para-a-elaboracao-e-apresentacao-das-demonstracoes-contabeis/microdados/'
   h <- try(rvest::read_html(url), silent = TRUE)
 
   # check if internet connection worked
@@ -120,6 +120,99 @@ get_airport_movement_dates_available <- function(date=NULL) {
 }
 
 
+#' Retrieve all dates available for airfares data from ANAC website
+#'
+#' @param dom Logical. Defaults to `TRUE` download airfares of domestic
+#'                 flights. If `FALSE`, the function downloads airfares of
+#'                 international flights.
+#'
+#' @return Numeric vector.
+#' @export
+#' @keywords internal
+#' @examples \dontrun{ if (interactive()) {
+#' # check dates
+#' a <- get_airfares_dates_available(domestic = TRUE)
+#'}}
+get_airfares_dates_available <- function(dom) {
+
+  if( ! is.logical(dom) ){ stop(paste0("Argument 'dom' must be either 'TRUE' or 'FALSE.")) }
+
+  # read html table
+  if( isTRUE(dom) ) { base_url = 'https://sistemas.anac.gov.br/sas/tarifadomestica/' }
+  if( isFALSE(dom)) { base_url = 'https://sistemas.anac.gov.br/sas/tarifainternacional/' }
+
+  h <- try(rvest::read_html(base_url), silent = TRUE)
+
+  # check if internet connection worked
+  if (class(h)[1]=='try-error') {                                           #nocov
+    message("Problem connecting to ANAC data server. Please try it again.") #nocov
+    return(invisible(NULL))                                                 #nocov
+  }
+
+  # filter elements of basica data
+  elements <- rvest::html_elements(h, "a")
+
+  if( isTRUE(dom) ) {
+    basica_urls <- elements[ data.table::like(elements, '/tarifadomestica/2') ]
+  }
+
+  if( isFALSE(dom)) {
+    basica_urls <- elements[ data.table::like(elements, '/tarifainternacional/2') ]
+  }
+
+
+  basica_urls <- lapply(X=basica_urls, FUN=function(i){rvest::html_attr(i,"href")})
+
+  # get all dates available
+  years <- gsub("[^\\d]+", "", basica_urls, perl=TRUE)
+
+  # get url of subdirectories
+  urls <- paste0(base_url, years)
+
+  # function to search .csv data in subdirectories
+  recursive_search <- function(i){ # i=urls[2]
+
+    # read html table
+    h2 <- try(rvest::read_html(i), silent = TRUE)
+
+    if (class(h2)[1]=='try-error') {
+      message("Problem connecting to ANAC data server. Please try it again.")
+      return(invisible(NULL))}
+
+    # get url of subdirectories
+    elements2 <- rvest::html_elements(h2, "a")
+    href2 <- rvest::html_attr(elements2, "href")
+    # files_all <- grep("../", href2, fixed = TRUE, value = TRUE, invert = TRUE)
+    files_csv <- href2[ data.table::like(href2, '.csv|.txt')]
+    # temp_urls <- paste0(i, files_csv)
+    # return(temp_urls)
+    return(files_csv)
+  }
+
+  # get urls of .csv files
+  csv_urls <- lapply(X=urls, FUN=recursive_search)
+  csv_urls <- unlist(csv_urls)
+
+  # get all dates available
+  options(warn=-1) # suppress warnings
+  if( isTRUE(dom) ) {
+    all_dates <- substr(csv_urls , (nchar(csv_urls ) + 1) -10, nchar(csv_urls )-4 ) }
+
+  if( isFALSE(dom)) {
+    csv_urls <- csv_urls[ nchar(csv_urls)==55 ]
+    all_dates <- substr(csv_urls , (nchar(csv_urls ) + 1) -11, nchar(csv_urls )-4 )
+    all_dates <- gsub("[-]", "", all_dates)
+    }
+
+  all_dates <- as.numeric(all_dates)
+  all_dates <- all_dates[ ! is.na(all_dates)]
+  options(warn=0) # unsuppress warnings
+
+  return(all_dates)
+}
+
+
+
 #' Check whether date input is acceptable
 #' @param date Numeric. Either a 6-digit date in the format `yyyymm` or a 4-digit
 #'             date input `yyyy` .
@@ -190,11 +283,11 @@ generate_all_months <- function(date) { # nocov start
 #'}}
 get_flights_url <- function(type, year, month) { # nocov start
 
-  # https://www.gov.br/anac/pt-br/assuntos/regulados/empresas-aereas/envio-de-informacoes/microdados/basica2021-01.zip
-
+  # old https://www.gov.br/anac/pt-br/assuntos/regulados/empresas-aereas/envio-de-informacoes/microdados/basica2021-01.zip
+  # new https://www.gov.br/anac/pt-br/assuntos/regulados/empresas-aereas/Instrucoes-para-a-elaboracao-e-apresentacao-das-demonstracoes-contabeis/microdados/basica2021-01.zip
   if( nchar(month) ==1 ) { month <- paste0('0', month)}
 
-  url_root <- 'https://www.gov.br/anac/pt-br/assuntos/regulados/empresas-aereas/envio-de-informacoes/microdados/'
+  url_root <- 'https://www.gov.br/anac/pt-br/assuntos/regulados/empresas-aereas/Instrucoes-para-a-elaboracao-e-apresentacao-das-demonstracoes-contabeis/microdados/'
   file_name <- paste0(type, year, '-', month, '.zip')
   file_url <- paste0(url_root, file_name)
   return(file_url)
@@ -202,6 +295,99 @@ get_flights_url <- function(type, year, month) { # nocov start
 
 
 
+
+#' Put together the url of airfare data files
+#'
+#' @param dom Logical. Defaults to `TRUE` download airfares of domestic
+#'                 flights. If `FALSE`, the function downloads airfares of
+#'                 international flights.
+#' @param year Numeric. Year of the data in `yyyy` format.
+#' @param month Numeric. Month of the data in `mm` format.
+#'
+#' @return A url string.
+#'
+#' @keywords internal
+#' @examples \dontrun{ if (interactive()) {
+#' # Generate url
+#' a <- get_airfares_url(year=2002, month=11)
+#'}}
+get_airfares_url <- function(dom, year, month) { # nocov start
+
+  # https://sistemas.anac.gov.br/sas/tarifadomestica/2006/200605.csv
+  # https://sistemas.anac.gov.br/sas/tarifainternacional/2015/Internacional_2015-04.txt
+
+  if( nchar(month) ==1 ) { month <- paste0('0', month)}
+
+  # put file url together
+  if( isTRUE(dom) ) {
+    url_root = 'https://sistemas.anac.gov.br/sas/tarifadomestica/'
+    file_name <- paste0(year, month, '.csv')
+    file_url <- paste0(url_root, year, '/', file_name)
+  }
+
+  if( isFALSE(dom)) {
+
+    url_root = 'https://sistemas.anac.gov.br/sas/tarifainternacional/'
+
+    if( year > 2016) { file_name <- paste0(year,'-', month, '.csv') } else {
+     file_name <- paste0(year,'-', month, '.txt')}
+
+    file_url <- paste0(url_root, year, '/Internacional_', file_name)
+    }
+
+  return(file_url)
+} # nocov end
+
+
+#' Download file from url
+#'
+#' @param file_url String. A url passed from get_flights_url.
+#' @param showProgress Logical, passed from \code{\link{read_flights}}
+#'
+#' @return Silently saves downloaded file to temp dir.
+#'
+#' @keywords internal
+#' @examples \dontrun{ if (interactive()) {
+#' # Generate url
+#' file_url <- get_flights_url(type='basica', year=2000, month=11)
+#'
+#' # download data
+#' download_flightsbr_file(file_url=file_url, showProgress=TRUE)
+#'}}
+download_flightsbr_file <- function(file_url, showProgress=showProgress){
+
+  # create temp local file
+  file_name <- basename(file_url)
+  temp_local_file <- paste0(tempdir(),"/",file_name)
+
+  # download data
+  try(
+    httr::GET(url=file_url,
+              if(showProgress==T){ httr::progress()},
+              httr::write_disk(temp_local_file, overwrite = T),
+              config = httr::config(ssl_verifypeer = FALSE)
+    ), silent = TRUE)
+
+  # check if file has NOT been downloaded, try a 2nd time
+  if (!file.exists(temp_local_file) | file.info(temp_local_file)$size == 0) {
+
+    # download data: try a 2nd time
+    try(
+      httr::GET(url=file_url,
+                if(showProgress==T){ httr::progress()},
+                httr::write_disk(temp_local_file, overwrite = T),
+                config = httr::config(ssl_verifypeer = FALSE)
+      ), silent = TRUE)
+  }
+
+  # Halt function if download failed
+  if (!file.exists(temp_local_file) | file.info(temp_local_file)$size == 0) {
+    message('Internet connection not working.')
+    return(invisible(NULL)) }
+}
+
+
+
 #' Download and read ANAC flight data
 #'
 #' @param file_url String. A url passed from get_flights_url.
@@ -221,44 +407,25 @@ get_flights_url <- function(type, year, month) { # nocov start
 download_flights_data <- function(file_url, showProgress=showProgress, select=select){ # nocov start
 
   # create temp local file
-  # file_name <- substr(file_url, (nchar(file_url) + 1) -17, nchar(file_url) )
   file_name <- basename(file_url)
-  temp_local_file <- tempfile( file_name )
+  temp_local_file <- paste0(tempdir(),"/",file_name)
+
+  # check if file has not been downloaded already. If not, download it
+  if (!file.exists(temp_local_file) | file.info(temp_local_file)$size == 0) {
 
   # download data
-  try(
-    httr::GET(url=file_url,
-              if(showProgress==T){ httr::progress()},
-              httr::write_disk(temp_local_file, overwrite = T),
-              config = httr::config(ssl_verifypeer = FALSE)
-    ), silent = TRUE)
-
-  # address of zipped file stored locally
-  temp_local_file_zip <- paste0('unzip -p ', temp_local_file)
-
-  # check if file has been downloaded, try a 2nd time
-    if (!file.exists(temp_local_file) | file.info(temp_local_file)$size == 0) {
-
-            # download data: try a 2nd time
-            try(
-              httr::GET(url=file_url,
-                        if(showProgress==T){ httr::progress()},
-                        httr::write_disk(temp_local_file, overwrite = T),
-                        config = httr::config(ssl_verifypeer = FALSE)
-              ), silent = TRUE)
-      }
-
-  # check if file has been downloaded
-  if (!file.exists(temp_local_file) | file.info(temp_local_file)$size == 0) {
-        message('Internet connection not working.')
-        return(invisible(NULL)) }
+  download_flightsbr_file(file_url=file_url, showProgress=showProgress)
+  }
 
   ### set threads for fread
   orig_threads <- data.table::getDTthreads()
   data.table::setDTthreads(percent = 100)
 
+  # address of zipped file stored locally
+  temp_local_file_zip <- paste0('unzip -p ', temp_local_file)
+
   # read zipped file stored locally
-  dt <- data.table::fread( cmd =  temp_local_file_zip, select=select)
+  dt <- data.table::fread( cmd =  temp_local_file_zip, select=select, colClasses = 'character')
 
   # return to original threads
   data.table::setDTthreads(orig_threads)
@@ -267,6 +434,52 @@ download_flights_data <- function(file_url, showProgress=showProgress, select=se
   } # nocov end
 
 
+
+
+#' Download and read ANAC air fares data
+#'
+#' @param file_url String. A url passed from get_flights_url.
+#' @param showProgress Logical, passed from \code{\link{read_flights}}
+#' @param select A vector of column names or numbers to keep, passed from \code{\link{read_flights}}
+#'
+#' @return A `"data.table" "data.frame"` object
+#'
+#' @keywords internal
+#' @examples \dontrun{ if (interactive()) {
+#' # Generate url
+#' file_url <- get_airfares_url(dom = TRUE, year=2002, month=11)
+#'
+#' # download data
+#' a <- download_airfares_data(file_url=file_url, showProgress=TRUE, select=NULL)
+#'}}
+download_airfares_data <- function(file_url, showProgress=showProgress, select=select){ # nocov start
+
+  # create temp local file
+  file_name <- basename(file_url)
+  temp_local_file <- paste0(tempdir(),"/",file_name)
+
+  # check if file has not been downloaded already. If not, download it
+  if (!file.exists(temp_local_file) | file.info(temp_local_file)$size == 0) {
+
+    # download data
+    download_flightsbr_file(file_url=file_url, showProgress=showProgress)
+  }
+
+  ### set threads for fread
+  orig_threads <- data.table::getDTthreads()
+  data.table::setDTthreads(percent = 100)
+
+  # read file stored locally
+  dt <- data.table::fread(input = temp_local_file, select=select, colClasses = 'character') # , dec = ','
+
+  # return to original threads
+  data.table::setDTthreads(orig_threads)
+
+  return(dt)
+} # nocov end
+
+
+
 #' Convert latitude and longitude columns to numeric
 #'
 #' @param df A data.frame internal to the `read_airport()` function.
@@ -315,7 +528,7 @@ latlon_to_numeric <- function(df){ # nocov start
 #' @keywords internal
 #' @examples \dontrun{ if (interactive()) {
 #' # Generate url
-#' a <- get_flights_url(type='basica', year=2000, month=11)
+#' a <- get_airport_movements_url(year=2000, month=11)
 #'}}
 get_airport_movements_url <- function(year, month) { # nocov start
 
@@ -348,32 +561,33 @@ get_airport_movements_url <- function(year, month) { # nocov start
 #'}}
 download_airport_movement_data <- function(file_url, showProgress=showProgress){ # nocov start
 
-  # create temp local file
-  # file_name <- substr(file_url, (nchar(file_url) + 1) -17, nchar(file_url) )
+  # # create temp local file
   file_name <- basename(file_url)
+  temp_local_file <- paste0(tempdir(),"/",file_name)
 
-  temp_local_file <- tempfile( file_name )
+  # check if file has not been downloaded already. If not, download it
+  if (!file.exists(temp_local_file) | file.info(temp_local_file)$size == 0) {
+
+    # download data
+    download_flightsbr_file(file_url=file_url, showProgress=showProgress)
+  }
 
   ### set threads for fread
   orig_threads <- data.table::getDTthreads()
   data.table::setDTthreads(percent = 100)
 
+  # # assign column classes
+  # dt_classes <- list(IDate=c('DT_PREVISTO', 'DT_CALCO', 'DT_TOQUE'),
+  #                    ITime =c('HH_PREVISTO', 'HH_CALCO', 'HH_TOQUE'))
+
   # download data and read .csv data file
-  dt <- try( data.table::fread(file_url, showProgress = showProgress), silent = TRUE)
+  dt <- data.table::fread(temp_local_file, showProgress = showProgress, colClasses = 'character')
+  # class(dt$DT_CALCO)
+  # class(dt$HH_PREVISTO)
 
-    # check if file has been downloaded, try a 2nd time
-    if (class(dt)[1]=='try-error') {
-      dt <- try( data.table::fread(file_url, showProgress = showProgress), silent = TRUE)
-      }
 
   # return to original threads
   data.table::setDTthreads(orig_threads)
 
-  # check if file has been downloaded
-  if (class(dt)[1]=='try-error') {
-    message("Problem connecting to ANAC data server. Please try again.")
-    return(invisible(NULL))
-  }
-
   return(dt)
 } # nocov end
